#ifndef NUMVECTOR_H
#define NUMVECTOR_H

#include <cassert>
#include "myvector.h"
class NumVector : public MyVector
{
public:

#ifdef FORCE_BASE_CLASS_CONSTRUCTORS
//  Forcing the derived class to use the base class constructors doesn't work with the move constructor and the move assignment.
//
//// http://stackoverflow.com/questions/12783643/force-a-derived-class-to-use-the-constructor-of-the-base-class
//// http://www.stroustrup.com/C++11FAQ.html#inheriting
//
    using MyVector::MyVector;// <-- Uses the constructors from MyVector.

#else
//  Forcing the derived class to use the base class constructors doesn't work with the move constructor and the move assignment.

    //! \brief Allocates a vector with @p n elements on the heap.
    //!
    //! \param[in] n vector length
    //! \warning Throws an exception in case of insufficient memory on the heap.
    //!

    explicit NumVector(const long long int n = 0) 	// Standard-, Parameterkonstruktor
        : MyVector(n)
    {  }

    //! \brief Allocates a vector with the same elements as vector @p orig and copies its elements.
    //!
    //! \param[in] orig original vector
    //! \warning Throws an exception in case of insufficient memory.
    //!
    explicit NumVector(const NumVector& orig)	            // Copykonstruktor
        : MyVector(orig)
    { }

    //! \brief Allocates a vector with the same elements as vector @p orig and copies its elements.
    //!
    //! \param[in] orig original vector
    //! \warning Throws an exception in case of insufficient memory.
    //!
    NumVector(NumVector&& orig)	            // Movekonstruktor
        : MyVector(std::move(orig))         //  move is needed !!
    { }

    //! \brief Allocates a vector with the same elements as initializer list @p s and copies its elements.
    //!
    //! \param[in] s initializer list
    //! \warning Throws an exception in case of insufficient memory.
    //!
    explicit NumVector(const std::initializer_list<double> s) // constructor via init list
        : MyVector(s)
    {  }

    NumVector& operator=(NumVector&& orig)            // move assignment
    {
        MyVector::operator=(std::move(orig));         //  move ist needed !!
        return *this;
    }
#endif

    /** Default destructor */
    virtual ~NumVector() {}


    //! \brief Adds vector @p rhs to the current instance.
    //!
    //! \param[in] rhs  vector
    //! \return    reference to current instance.
    //! \warning Stops in case of unequal number of elements.
    //!
    NumVector& operator+=(const NumVector& rhs)
    {
        assert( size()==rhs.size() );
        for (long long k=0; k<size(); ++k)
        {
            val_[k] += rhs[k];
        }
        return *this;
    }

    //! \brief Subtracts vector @p rhs to the current instance.
    //!
    //! \param[in] rhs  vector
    //! \return    reference to current instance.
    //! \warning Stops in case of unequal number of elements.
    //!
    NumVector& operator-=(const NumVector& rhs)
    {
        assert( size()==rhs.size() );
        for (long long k=0; k<size(); ++k)
        {
            val_[k] -= rhs[k];
        }
        return *this;
    }


    //! \brief Scales the elements of the current instance by scalar @p alfa.
    //!
    //! \param[in] alfa  scalar
    //! \return    scaled current instance.
    //!
    NumVector& operator*=(const double alfa)
    {
        for (long long k=0; k<size(); ++k)
        {
            val_[k] *= alfa;
        }
        return *this;
    }


protected:
private:
};


//! \brief Adds two vectors @p a and @p b and returns the result.
//!
//! \param[in] a  first vector
//! \param[in] b  second vector
//! \return    vector of summed elements.
//! \warning Stops in case of unequal number of elements.
//!
NumVector operator+(const NumVector& a, const NumVector& b);       // could be also inlines


//! \brief Adds two vectors @p a and @p b and returns the result.
//!
//! \param[in] a  first vector
//! \param[in] b  second vector
//! \return    vector of summed elements.
//! \warning Stops in case of unequal number of elements.
//!
NumVector operator+(NumVector&& a, NumVector&& b);       // could be also inlines


inline
NumVector operator+(const NumVector& a, NumVector&& b)
{
    std::cout << "Re   RV\n";
    return std::move(b+=a);
}

inline
NumVector operator+(NumVector&& a, const NumVector& b)
{
    std::cout << "RV   Re\n";
    return std::move(a+=b);
}

/////////////////////////////////////////////////////////////////
//! \brief Substracts two vectors @p a and @p b and returns the result.
//!
//! \param[in] a  first vector
//! \param[in] b  second vector
//! \return    vector of subtracted elements.
//! \warning Stops in case of unequal number of elements.
//!
inline
NumVector operator-(const NumVector& a, const NumVector& b)
{
    NumVector tmp(a);
    return std::move(tmp-=b);
}

inline
NumVector operator-(NumVector&& a, NumVector&& b)
{
    return std::move(a-=b);
}

inline
NumVector operator-(const NumVector& a, NumVector&& b)
{
    NumVector tmp(a);
    return std::move(tmp-=b);
}

inline
NumVector operator-(NumVector&& a, const NumVector& b)
{
    std::cout << "RV   Re\n";
    return std::move(a-=b);
}

/////////////////////////////////////////////////////////////////
inline
NumVector operator*(const double alfa, const NumVector& b)
{
    NumVector tmp(b);
    return std::move(tmp*=alfa);
}

inline
NumVector operator*(const NumVector& b, const double alfa)
{
    NumVector tmp(b);
    return std::move(tmp*=alfa);
}

inline
NumVector operator*(const double alfa, NumVector&& b)
{
    return std::move(b*=alfa);
}

inline
NumVector operator*(NumVector&& b, const double alfa)
{
    return std::move(b*=alfa);
}


//! \brief Inner product of two vectors @p a and @p b.
//!
//! \param[in] a  first vector
//! \param[in] b  second vector
//! \return    inner product.
//! \warning Stops in case of unequal number of elements.
//!
inline
double innerProd(const NumVector& a, const NumVector& b)
{
    assert( a.size()==b.size() );
    double s(0.0);
    for (long long k=0; k<a.size(); ++k)
    {
        s += a[k]*b[k];
    }
    return s;
}

#endif // NUMVECTOR_H
